/* PDF image extractor based on PDFlib TET
 *
 * $Id: image_extractor.java,v 1.1 2013/05/13 14:13:37 rp Exp $
 */

import com.pdflib.TETException;
import com.pdflib.TET;

public class image_extractor
{
    /**
     * Global option list
     */
    static final String globaloptlist =
	    "searchpath={{../data}}";
    
    /**
     * Document-specific option list
     */
    static final String docoptlist = "";
    
    /**
     * Page-specific option list;
     * since we extract only images, we disable the text engine
     */
    static final String pageoptlist = "skipengines={text}";

    /* here you can insert basic image extract options (more below) */
    static final String baseimageoptlist = "";

    /**
     * Separator to emit after each chunk of text. This depends on the
     * applications needs; for granularity=word a space character may be useful.
     */
    static final String separator = "\n";

    public static void main (String argv[])
    {
        TET tet = null;
        
	try
        {
	    if (argv.length != 1)
            {
                throw new Exception(
		    "usage: extractor <filename>");
            }
	    String outfilebase = argv[0];

            tet = new TET();

            tet.set_option(globaloptlist);

            int doc = tet.open_document(outfilebase, docoptlist);

            if (doc == -1)
            {
                throw new Exception("Error " + tet.get_errnum() + "in "
                        + tet.get_apiname() + "(): " + tet.get_errmsg());
            }
            
            /* Get number of pages in the document */
            int n_pages = (int) tet.pcos_get_number(doc, "length:pages");

            /* loop over pages in the document */
            for (int pageno = 1; pageno <= n_pages; ++pageno)
            {
                String text;
                int page;
		
		page = tet.open_page(doc, pageno, pageoptlist);

                if (page == -1)
                {
                    print_tet_error(tet, pageno);
                    continue; /* try next page */
                }

                /*
                 * Retrieve all images on the page 
		 */
                while ((tet.get_image_info(page)) == 1)
                {
		    /* Report image geometry */
		    System.out.println("page " + pageno + ": " + 
			(int) tet.width + "x" +
			(int) tet.height + "pt, alpha=" + 
			tet.alpha + ", beta=" + 
			tet.beta);

		    /* Retrieve additional image properties with pCOS */
		    System.out.print("   id=" + tet.imageid + ", " +
			(int) tet.pcos_get_number(doc, "images[" + tet.imageid + "]/Width")+ "x" +
			(int) tet.pcos_get_number(doc, "images[" + tet.imageid + "]/Height")+ " pixel, ");
		    int cs = (int) tet.pcos_get_number(doc, "images["+ tet.imageid +
			"]/colorspaceid");

		    if (cs != -1){
		    System.out.println(
			(int) tet.pcos_get_number(doc, "colorspaces[" + cs + "]/components") + "x" +
			(int) tet.pcos_get_number(doc, "images[" + tet.imageid + "]/bpc") + " bit " +
			tet.pcos_get_string(doc, "colorspaces[" + cs + "]/name"));
		    }
		    else
		    {
			/* cs==-1 may happen for some JPEG 2000 images. bpc,
			 * colorspace name and number of components are not
			 * available in this case.  
			 */
										                        System.out.println("JPEG2000\n");
		    }
		    /*
		     * Fetch image data and write it to a disk file. The
		     * output filename is generated from the input filename,
		     * page number and image ID.
		     */

		    String imageoptlist = baseimageoptlist + " filename {" + 
			outfilebase + "_p" + pageno + "_I" + tet.imageid + "}";
		    if (tet.write_image_file(doc, tet.imageid, imageoptlist) == -1){
			System.out.println("Error " + tet.get_errnum() + " in " +
			    tet.get_apiname() + "(): " + tet.get_errmsg());
		    }

                }


                if (tet.get_errnum() != 0)
                {
                    print_tet_error(tet, pageno);
                }

                tet.close_page(page);
            }

            tet.close_document(doc);
        }
	catch (TETException e)
	{
	    System.err.println("TET exception occurred in extractor sample:");
	    System.err.println("[" + e.get_errnum() + "] " + e.get_apiname() +
			    ": " + e.get_errmsg());
        }
        catch (Exception e)
        {
            System.err.println(e.getMessage());
        }
        finally
        {
            if (tet != null) {
		tet.delete();
            }
        }
    }

    /**
     * Report a TET error.
     * 
     * @param tet The TET object
     * @param pageno The page number on which the error occurred
     */
    private static void print_tet_error(TET tet, int pageno)
    {
        System.err.println("Error " + tet.get_errnum() + " in  "
                + tet.get_apiname() + "() on page " + pageno + ": "
                + tet.get_errmsg());
    }
}
